@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2017, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -9,9 +11,10 @@
 #ifndef XFRAME_XFRAME_EXPRESSION_HPP
 #define XFRAME_XFRAME_EXPRESSION_HPP
 
-#include "xtensor/xexpression.hpp"
 #include "xtl/xtype_traits.hpp"
 
+#include "xtensor/xexpression.hpp"
+
 namespace xf
 {
     struct xvariable_expression_tag {};
@@ -40,4 +43,28 @@ namespace xf
     };
 }
 
+namespace xt
+{
+    namespace extension
+    {
+        template <>
+        struct expression_tag_and<xf::xvariable_expression_tag, xoptional_expression_tag>
+        {
+            using type = xf::xvariable_expression_tag;
+        };
+
+        template <>
+        struct expression_tag_and<xoptional_expression_tag, xf::xvariable_expression_tag>
+            : expression_tag_and<xf::xvariable_expression_tag, xoptional_expression_tag>
+        {
+        };
+
+        template <>
+        struct expression_tag_and<xf::xvariable_expression_tag, xf::xvariable_expression_tag>
+        {
+            using type = xf::xvariable_expression_tag;
+        };
+    }
+}
+
 #endif
