@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2017, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -12,18 +14,16 @@
 #include "xtensor/xexpression.hpp"
 
 #include "xcoordinate_chain.hpp"
+#include "xreindex_data.hpp"
 #include "xvariable.hpp"
 
 namespace xf
 {
-    
+
     /*****************
      * xreindex_view *
      *****************/
 
-    template <class CT>
-    class xreindex_view;
-
     template <class CT>
     class xreindex_view : public xt::xexpression<xreindex_view<CT>>
     {
@@ -38,8 +38,9 @@ namespace xf
         using const_pointer = pointer;
         using size_type = typename xexpression_type::size_type;
         using difference_type = typename xexpression_type::difference_type;
-        
+
         using shape_type = typename xexpression_type::shape_type;
+        using data_type = xreindex_data<self_type>;
 
         using subcoordinate_type = typename xexpression_type::coordinate_type;
         using coordinate_type = xcoordinate_chain<subcoordinate_type>;
@@ -68,9 +69,12 @@ namespace xf
 
         static const_reference missing();
 
+        xreindex_view(self_type&& rhs);
+        xreindex_view(const self_type& rhs);
+
         template <class E>
         xreindex_view(E&& e, const coordinate_map& new_coord);
-        
+
         template <class E>
         xreindex_view(E&& e, coordinate_map&& new_coord);
 
@@ -80,11 +84,12 @@ namespace xf
         const coordinate_type& coordinates() const noexcept;
         const dimension_type& dimension_mapping() const noexcept;
 
-        template <class Join = DEFAULT_JOIN, class C = coordinate_type>
+        template <class Join = XFRAME_DEFAULT_JOIN, class C = coordinate_type>
         xtrivial_broadcast broadcast_coordinates(C& coords) const;
         bool broadcast_dimensions(dimension_type& dims, bool trivial_bc = false) const;
 
         const shape_type& shape() const noexcept;
+        const data_type& data() const noexcept;
 
         template <class... Args>
         const_reference operator()(Args... args) const;
@@ -94,20 +99,20 @@ namespace xf
 
         template <std::size_t N = dynamic()>
         const_reference element(index_type<N>&& index) const;
-    
+
         template <class... Args>
         const_reference locate(Args&&... args) const;
-    
+
         template <std::size_t N = dynamic()>
         const_reference locate_element(const locator_sequence_type<N>& locator) const;
 
         template <std::size_t N = dynamic()>
         const_reference locate_element(locator_sequence_type<N>&& locator) const;
 
-        template <class Join = DEFAULT_JOIN, std::size_t N = std::numeric_limits<size_type>::max()>
+        template <class Join = XFRAME_DEFAULT_JOIN, std::size_t N = std::numeric_limits<size_type>::max()>
         const_reference select(const selector_sequence_type<N>& selector) const;
 
-        template <class Join = DEFAULT_JOIN, std::size_t N = dynamic()>
+        template <class Join = XFRAME_DEFAULT_JOIN, std::size_t N = dynamic()>
         const_reference select(selector_sequence_type<N>&& selector) const;
 
         template <std::size_t N = dynamic()>
@@ -142,6 +147,7 @@ namespace xf
         coordinate_type m_coordinate;
         const dimension_type& m_dimension_mapping;
         shape_type m_shape;
+        data_type m_data;
     };
 
     template <class CT>
@@ -160,16 +166,40 @@ namespace xf
     template <class E1, class E2>
     auto reindex_like(E1&& e1, const E2& e2);
 
+    template <class Join, class E1, class... E>
+    auto align(E1&& e1, E&&... e);
+
     /********************************
-     * xreindex_view implmeentation *
+     * xreindex_view implementation *
      ********************************/
 
+    template <class CT>
+    inline xreindex_view<CT>::xreindex_view(self_type&& rhs)
+        : m_e(std::forward<decltype(rhs.m_e)>(rhs.m_e)),
+          m_coordinate(std::move(rhs.m_coordinate)),
+          m_dimension_mapping(m_e.dimension_mapping()),
+          m_data(*this)
+    {
+        init_shape();
+    }
+
+    template <class CT>
+    inline xreindex_view<CT>::xreindex_view(const self_type& rhs)
+        : m_e(rhs.m_e),
+          m_coordinate(rhs.m_coordinate),
+          m_dimension_mapping(m_e.dimension_mapping()),
+          m_data(*this)
+    {
+        init_shape();
+    }
+
     template <class CT>
     template <class E>
     inline xreindex_view<CT>::xreindex_view(E&& e, const coordinate_map& new_coord)
         : m_e(std::forward<E>(e)),
           m_coordinate(reindex(m_e.coordinates(), new_coord)),
-          m_dimension_mapping(m_e.dimension_mapping())
+          m_dimension_mapping(m_e.dimension_mapping()),
+          m_data(*this)
     {
         init_shape();
     }
@@ -179,7 +209,8 @@ namespace xf
     inline xreindex_view<CT>::xreindex_view(E&& e, coordinate_map&& new_coord)
         : m_e(std::forward<E>(e)),
           m_coordinate(reindex(m_e.coordinates(), std::move(new_coord))),
-          m_dimension_mapping(m_e.dimension_mapping())
+          m_dimension_mapping(m_e.dimension_mapping()),
+          m_data(*this)
     {
         init_shape();
     }
@@ -249,6 +280,12 @@ namespace xf
         return m_shape;
     }
 
+    template <class CT>
+    inline auto xreindex_view<CT>::data() const noexcept -> const data_type&
+    {
+        return m_data;
+    }
+
     template <class CT>
     template <class... Args>
     inline auto xreindex_view<CT>::operator()(Args... args) const -> const_reference
@@ -271,7 +308,7 @@ namespace xf
     {
         return element_impl<N>(std::move(index));
     }
-    
+
     template <class CT>
     template <class... Args>
     inline auto xreindex_view<CT>::locate(Args&&... args) const -> const_reference
@@ -280,7 +317,7 @@ namespace xf
         using loc_value_type = typename locator_sequence_type<N>::value_type;
         return locate_element_impl<N>(locator_sequence_type<N>{loc_value_type(args)...});
     }
-    
+
     template <class CT>
     template <std::size_t N>
     inline auto xreindex_view<CT>::locate_element(const locator_sequence_type<N>& locator) const -> const_reference
@@ -322,7 +359,7 @@ namespace xf
     {
         return iselect_impl<N>(std::move(selector));
     }
-    
+
     template <class CT>
     inline void xreindex_view<CT>::init_shape()
     {
@@ -333,7 +370,7 @@ namespace xf
             m_shape[i] = coordinates().find(dimension_labels()[i])->second.size();
         }
     }
-   
+
     template <class CT>
     template <std::size_t N, class IDX>
     inline auto xreindex_view<CT>::element_impl(IDX&& index) const -> const_reference
@@ -378,7 +415,7 @@ namespace xf
         }
         return m_e.template locate_element<N>(std::forward<L>(locator));
     }
-    
+
     template <class CT>
     template <class S>
     inline auto xreindex_view<CT>::select_impl(S&& selector) const -> const_reference
@@ -466,22 +503,46 @@ namespace xf
         return view_type(std::forward<E>(e), std::move(new_coord));
     }
 
+    namespace detail
+    {
+        template <class E1, class C>
+        inline auto reindex_like_coord(E1&& e1, const C& coords)
+        {
+            using view_type = xreindex_view<xtl::closure_type_t<E1>>;
+            using coordinate_map = typename view_type::coordinate_map;
+
+            coordinate_map new_coord;
+            for(auto iter = e1.coordinates().begin(); iter != e1.coordinates().end(); ++iter)
+            {
+                const auto& axis = coords[iter->first];
+                if(axis != iter->second)
+                {
+                    new_coord.insert(std::make_pair(iter->first, axis));
+                }
+            }
+            return view_type(std::forward<E1>(e1), std::move(new_coord));
+        }
+    }
+
     template <class E1, class E2>
     inline auto reindex_like(E1&& e1, const E2& e2)
     {
-        using view_type = xreindex_view<xtl::closure_type_t<E1>>;
-        using coordinate_map = typename view_type::coordinate_map;
+        return detail::reindex_like_coord(std::forward<E1>(e1), e2.coordinates());
+    }
 
-        coordinate_map new_coord;
-        for(auto iter = e1.coordinates().begin(); iter != e1.coordinates().end(); ++iter)
-        {
-            const auto& axis = e2.coordinates()[iter->first];
-            if(axis != iter->second)
-            {
-                new_coord.insert(std::make_pair(iter->first, axis));
-            }
-        }
-        return view_type(std::forward<E1>(e1), std::move(new_coord));
+    template <class Join, class E1, class... E>
+    inline auto align(E1&& e1, E&&... e)
+    {
+        auto coord = e1.coordinates();
+        broadcast_coordinates<Join>(coord, e.coordinates()...);
+        return std::make_tuple(detail::reindex_like_coord(std::forward<E1>(e1), coord),
+                               detail::reindex_like_coord(std::forward<E>(e), coord)...);
+    }
+
+    template <class CT>
+    inline std::ostream& operator<<(std::ostream& out, const xreindex_view<CT>& view)
+    {
+        return print_variable_expression(out, view);
     }
 }
 
