@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2017, Johan Mabille, Sylvain Corlay and Wolf Vollprecht    *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -35,6 +37,7 @@ namespace xf
         using dimension_closure_type = dimension_type;
         using key_type = typename coordinate_type::key_type;
         using size_type = typename coordinate_type::size_type;
+        using label_list = typename coordinate_type::label_list;
     };
 }
 
@@ -47,9 +50,11 @@ namespace xt
         using base_type = xf::xvariable_inner_types<xf::xvariable_view<CT>>;
         using xexpression_type = typename base_type::xexpression_type;
         using optional_type = typename xexpression_type::value_type;
-        using temporary_coordinate_type = xf::xcoordinate<typename base_type::key_type, typename base_type::size_type>;
+        using temporary_coordinate_type = xf::xcoordinate<typename base_type::key_type,
+                                                          typename base_type::label_list,
+                                                          typename base_type::size_type>;
         using temporary_data_type = xoptional_assembly<xarray<typename optional_type::value_type>, xarray<bool>>;
-        using temporary_type = xf::xvariable<temporary_coordinate_type, temporary_data_type>;
+        using temporary_type = xf::xvariable_container<temporary_coordinate_type, temporary_data_type>;
     };
 }
 
@@ -176,13 +181,13 @@ namespace xf
         template <std::size_t N = dynamic()>
         reference select(const selector_sequence_type<N>& selector);
 
-        template <class Join = DEFAULT_JOIN, std::size_t N = std::numeric_limits<size_type>::max()>
+        template <class Join = XFRAME_DEFAULT_JOIN, std::size_t N = std::numeric_limits<size_type>::max()>
         const_reference select(const selector_sequence_type<N>& selector) const;
 
         template <std::size_t N = dynamic()>
         reference select(selector_sequence_type<N>&& selector);
 
-        template <class Join = DEFAULT_JOIN, std::size_t N = dynamic()>
+        template <class Join = XFRAME_DEFAULT_JOIN, std::size_t N = dynamic()>
         const_reference select(selector_sequence_type<N>&& selector) const;
 
         template <std::size_t N = dynamic()>
@@ -274,10 +279,10 @@ namespace xf
     template <class E, class... S>
     auto ilocate(E&& e, S&&... slices);
 
-    template <class L = DEFAULT_LABEL_LIST, class E, class... S>
+    template <class L = XFRAME_DEFAULT_LABEL_LIST, class E, class... S>
     auto locate(E&& e, S&&... slices);
 
-    template <class E, class L = DEFAULT_LABEL_LIST>
+    template <class E, class L = XFRAME_DEFAULT_LABEL_LIST>
     auto select(E&& e, std::map<typename std::decay_t<E>::key_type, xaxis_slice<L>>&& slices);
 
     template <class E, class T = typename std::decay_t<E>::difference_type>
